@@ -34,7 +34,7 @@ from neutron._i18n import _
 from neutron.agent.common import ip_lib
 from neutron.agent.common import utils
 from neutron.agent.ovsdb import impl_idl
-from neutron.common import constants as common_constants
+from neutron.common import _constants as common_constants
 from neutron.common import utils as common_utils
 from neutron.conf.agent import ovs_conf
 from neutron.plugins.ml2.drivers.openvswitch.agent.common \
@@ -90,7 +90,7 @@ def _ovsdb_retry(fn):
         new_fn = tenacity.retry(
             reraise=True,
             retry=tenacity.retry_if_result(_ovsdb_result_pending),
-            wait=tenacity.wait_exponential(multiplier=0.01, max=1),
+            wait=tenacity.wait_exponential(multiplier=0.02, max=1),
             stop=tenacity.stop_after_delay(
                 self.ovsdb_timeout))(fn)
         return new_fn(*args, **kwargs)
@@ -118,7 +118,6 @@ class BaseOVS(object):
     def __init__(self):
         self.ovsdb_timeout = cfg.CONF.OVS.ovsdb_timeout
         self.ovsdb = impl_idl.api_factory()
-        self._hw_offload = None
 
     def add_manager(self, connection_uri, timeout=_SENTINEL):
         """Have ovsdb-server listen for manager connections
@@ -199,13 +198,6 @@ class BaseOVS(object):
         _cfg = self.config
         return {k: _cfg.get(k, OVS_DEFAULT_CAPS[k]) for k in OVS_DEFAULT_CAPS}
 
-    @property
-    def is_hw_offload_enabled(self):
-        if self._hw_offload is None:
-            self._hw_offload = self.config.get('other_config',
-                                   {}).get('hw-offload', '').lower() == 'true'
-        return self._hw_offload
-
 
 # Map from version string to on-the-wire protocol version encoding:
 OF_PROTOCOL_TO_VERSION = {
@@ -274,18 +266,6 @@ class OVSBridge(BaseOVS):
                                             protocol,
                                             key=version_from_protocol)
 
-    def set_igmp_snooping_state(self, state):
-        state = bool(state)
-        other_config = {
-            'mcast-snooping-disable-flood-unregistered': str(state)}
-        with self.ovsdb.transaction() as txn:
-            txn.add(
-                self.ovsdb.db_set('Bridge', self.br_name,
-                                  ('mcast_snooping_enable', state)))
-            txn.add(
-                self.ovsdb.db_set('Bridge', self.br_name,
-                                  ('other_config', other_config)))
-
     def create(self, secure_mode=False):
         other_config = {
             'mac-table-size': str(cfg.CONF.OVS.bridge_mac_table_size)}
@@ -339,7 +319,7 @@ class OVSBridge(BaseOVS):
             # TODO(mangelajo): We could accept attr tuples for the Port too
             # but, that could potentially break usage of this function in
             # stable branches (where we need to backport).
-            # https://review.openstack.org/#/c/564825/4/neutron/agent/common/
+            # https://review.opendev.org/#/c/564825/4/neutron/agent/common/
             # ovs_lib.py@289
             if interface_attr_tuples:
                 txn.add(self.ovsdb.db_set('Interface', port_name,
@@ -392,24 +372,6 @@ class OVSBridge(BaseOVS):
                           port_name)
         return ofport
 
-    def get_port_external_ids(self, port_name):
-        """Get the port's assigned ofport, retrying if not yet assigned."""
-        port_external_ids = dict()
-        try:
-            port_external_ids = self._get_port_val(port_name, "external_ids")
-        except tenacity.RetryError:
-            LOG.exception("Timed out retrieving external_ids on port %s.",
-                          port_name)
-        return port_external_ids
-
-    def get_port_mac(self, port_name):
-        """Get the port's mac address.
-
-        This is especially useful when the port is not a neutron port.
-        E.g. networking-sfc needs the MAC address of "patch-tun
-        """
-        return self.db_get_val("Interface", port_name, "mac_in_use")
-
     @_ovsdb_retry
     def _get_datapath_id(self):
         return self.db_get_val('Bridge', self.br_name, 'datapath_id')
@@ -528,11 +490,7 @@ class OVSBridge(BaseOVS):
         options['local_ip'] = local_ip
         options['in_key'] = 'flow'
         options['out_key'] = 'flow'
-        # NOTE(moshele): pkt_mark is not upported when using ovs hw-offload,
-        # therefore avoid clear mark on encapsulating packets when it's
-        # enabled
-        if not self.is_hw_offload_enabled:
-            options['egress_pkt_mark'] = '0'
+        options['egress_pkt_mark'] = '0'
         if tunnel_csum:
             options['csum'] = str(tunnel_csum).lower()
         if tos:
@@ -720,8 +678,8 @@ class OVSBridge(BaseOVS):
         self.set_controller_field('inactivity_probe', interval * 1000)
 
     def _set_egress_bw_limit_for_port(self, port_name, max_kbps,
-                                      max_burst_kbps, check_error=True):
-        with self.ovsdb.transaction(check_error=check_error) as txn:
+                                      max_burst_kbps):
+        with self.ovsdb.transaction(check_error=True) as txn:
             txn.add(self.ovsdb.db_set('Interface', port_name,
                                       ('ingress_policing_rate', max_kbps)))
             txn.add(self.ovsdb.db_set('Interface', port_name,
@@ -748,7 +706,8 @@ class OVSBridge(BaseOVS):
     def delete_egress_bw_limit_for_port(self, port_name):
         if not self.port_exists(port_name):
             return
-        self._set_egress_bw_limit_for_port(port_name, 0, 0, check_error=False)
+        self._set_egress_bw_limit_for_port(
+            port_name, 0, 0)
 
     def find_qos(self, port_name):
         qos = self.ovsdb.db_find(
@@ -857,8 +816,8 @@ class OVSBridge(BaseOVS):
 
     def update_ingress_bw_limit_for_port(self, port_name, max_kbps,
                                          max_burst_kbps):
-        max_bw_in_bits = max_kbps * common_constants.SI_BASE
-        max_burst_in_bits = max_burst_kbps * common_constants.SI_BASE
+        max_bw_in_bits = max_kbps * p_const.SI_BASE
+        max_burst_in_bits = max_burst_kbps * p_const.SI_BASE
         port_type = self._get_port_val(port_name, "type")
         if port_type in constants.OVS_DPDK_PORT_TYPES:
             self._update_ingress_bw_limit_for_dpdk_port(
@@ -878,18 +837,18 @@ class OVSBridge(BaseOVS):
             other_config = qos_res['other_config']
             max_bw_in_bits = other_config.get('max-rate')
             if max_bw_in_bits is not None:
-                qos_max_kbps = int(max_bw_in_bits) / common_constants.SI_BASE
+                qos_max_kbps = int(max_bw_in_bits) / p_const.SI_BASE
 
         queue_res = self.find_queue(port_name, QOS_DEFAULT_QUEUE)
         if queue_res:
             other_config = queue_res['other_config']
             max_bw_in_bits = other_config.get('max-rate')
             if max_bw_in_bits is not None:
-                queue_max_kbps = int(max_bw_in_bits) / common_constants.SI_BASE
+                queue_max_kbps = int(max_bw_in_bits) / p_const.SI_BASE
             max_burst_in_bits = other_config.get('burst')
             if max_burst_in_bits is not None:
                 max_burst_kbit = (
-                    int(max_burst_in_bits) / common_constants.SI_BASE)
+                    int(max_burst_in_bits) / p_const.SI_BASE)
 
         if qos_max_kbps == queue_max_kbps:
             max_kbps = qos_max_kbps
@@ -910,25 +869,25 @@ class OVSBridge(BaseOVS):
             if max_bw_in_bytes is not None:
                 max_kbps = common_utils.bits_to_kilobits(
                     common_utils.bytes_to_bits(int(float(max_bw_in_bytes))),
-                    common_constants.SI_BASE)
+                    p_const.SI_BASE)
             max_burst_in_bytes = other_config.get("cbs")
             if max_burst_in_bytes is not None:
                 max_burst_kbit = common_utils.bits_to_kilobits(
                     common_utils.bytes_to_bits(int(float(max_burst_in_bytes))),
-                    common_constants.SI_BASE)
+                    p_const.SI_BASE)
         return max_kbps, max_burst_kbit
 
     def delete_ingress_bw_limit_for_port(self, port_name):
-        self.ovsdb.db_clear('Port', port_name,
-                            'qos').execute(check_error=False)
         qos = self.find_qos(port_name)
         queue = self.find_queue(port_name, QOS_DEFAULT_QUEUE)
-        if qos:
-            self.ovsdb.db_destroy('QoS',
-                                  qos['_uuid']).execute(check_error=True)
-        if queue:
-            self.ovsdb.db_destroy('Queue',
-                                  queue['_uuid']).execute(check_error=True)
+        does_port_exist = self.port_exists(port_name)
+        with self.ovsdb.transaction(check_error=True) as txn:
+            if does_port_exist:
+                txn.add(self.ovsdb.db_clear("Port", port_name, 'qos'))
+            if qos:
+                txn.add(self.ovsdb.db_destroy('QoS', qos['_uuid']))
+            if queue:
+                txn.add(self.ovsdb.db_destroy('Queue', queue['_uuid']))
 
     def set_controller_field(self, field, value):
         attr = [(field, value)]
