@@ -17,12 +17,12 @@ import os
 import threading
 
 import eventlet
+from neutron_lib import constants
 from oslo_log import log as logging
 from oslo_utils import fileutils
 import webob
 
 from neutron.agent.linux import utils as agent_utils
-from neutron.common import constants
 from neutron.notifiers import batch_notifier
 
 LOG = logging.getLogger(__name__)
@@ -153,14 +153,16 @@ class AgentMixin(object):
             # same as the "transition state" (e.g.: backup-->master-->backup).
             return
 
-        state_change_data = {"router_id": router_id, "state": state}
-        LOG.info('Router %(router_id)s transitioned to %(state)s',
-                 state_change_data)
-
         ri = self._get_router_info(router_id)
         if ri is None:
             return
 
+        state_change_data = {"router_id": router_id, "state": state,
+                             "host": ri.agent.host}
+        LOG.info('Router %(router_id)s transitioned to %(state)s on '
+                 'agent %(host)s',
+                 state_change_data)
+
         # TODO(dalvarez): Fix bug 1677279 by moving the IPv6 parameters
         # configuration to keepalived-state-change in order to remove the
         # dependency that currently exists on l3-agent running for the IPv6
