@@ -24,6 +24,7 @@ from neutron_lib import constants
 from neutron_lib import context
 from neutron_lib import exceptions
 from neutron_lib.plugins import directory
+from neutron_lib.services import constants as service_const
 from oslo_config import cfg
 from oslo_db import exception as db_exc
 from oslo_log import log as logging
@@ -34,7 +35,6 @@ import stevedore
 
 from neutron._i18n import _
 from neutron.common import cache_utils as cache
-from neutron.common import constants as const
 
 
 LOG = logging.getLogger(__name__)
@@ -45,8 +45,7 @@ ADVSVC_CTX_POLICY = 'context_is_advsvc'
 
 # Identify the attribute used by a resource to reference another resource
 _RESOURCE_FOREIGN_KEYS = {
-    net_apidef.COLLECTION_NAME: 'network_id',
-    'security_groups': 'security_group_id'
+    net_apidef.COLLECTION_NAME: 'network_id'
 }
 
 
@@ -108,11 +107,11 @@ def set_rules(policies, overwrite=True):
 
 def _is_attribute_explicitly_set(attribute_name, resource, target, action):
     """Verify that an attribute is present and is explicitly set."""
-    if target.get(const.ATTRIBUTES_TO_UPDATE):
+    if target.get(constants.ATTRIBUTES_TO_UPDATE):
         # In the case of update, the function should not pay attention to a
         # default value of an attribute, but check whether it was explicitly
         # marked as being updated instead.
-        return (attribute_name in target[const.ATTRIBUTES_TO_UPDATE] and
+        return (attribute_name in target[constants.ATTRIBUTES_TO_UPDATE] and
                 target[attribute_name] is not constants.ATTR_NOT_SPECIFIED)
     result = (attribute_name in target and
               target[attribute_name] is not constants.ATTR_NOT_SPECIFIED)
@@ -260,9 +259,9 @@ class OwnerCheck(policy.Check):
         # having a way to map resources to plugins so to make this
         # check more general
         plugin = directory.get_plugin()
-        if resource_type in const.EXT_PARENT_RESOURCE_MAPPING:
+        if resource_type in service_const.EXT_PARENT_RESOURCE_MAPPING:
             plugin = directory.get_plugin(
-                const.EXT_PARENT_RESOURCE_MAPPING[resource_type])
+                service_const.EXT_PARENT_RESOURCE_MAPPING[resource_type])
         f = getattr(plugin, 'get_%s' % resource_type)
         # f *must* exist, if not found it is better to let neutron
         # explode. Check will be performed with admin context
@@ -311,9 +310,9 @@ class OwnerCheck(policy.Check):
                     reason=err_reason)
             parent_foreign_key = _RESOURCE_FOREIGN_KEYS.get(
                 "%ss" % parent_res, None)
-            if parent_res == const.EXT_PARENT_PREFIX:
-                for resource in const.EXT_PARENT_RESOURCE_MAPPING:
-                    key = "%s_%s_id" % (const.EXT_PARENT_PREFIX, resource)
+            if parent_res == constants.EXT_PARENT_PREFIX:
+                for resource in service_const.EXT_PARENT_RESOURCE_MAPPING:
+                    key = "%s_%s_id" % (constants.EXT_PARENT_PREFIX, resource)
                     if key in target:
                         parent_foreign_key = key
                         parent_res = resource
