@@ -13,7 +13,6 @@
 #    under the License.
 
 from neutron_lib import constants
-from neutronclient.common import exceptions as nc_exc
 from oslo_utils import uuidutils
 
 from neutron.cmd.sanity import checks
@@ -41,13 +40,10 @@ class OVSVersionChecker(object):
 
 class BaseSecurityGroupsSameNetworkTest(base.BaseFullStackTestCase):
 
-    of_interface = None
-
     def setUp(self):
         debug_iptables = self.firewall_driver.startswith("iptables")
         host_descriptions = [
             environment.HostDescription(
-                of_interface=self.of_interface,
                 l2_agent_type=self.l2_agent_type,
                 firewall_driver=self.firewall_driver,
                 dhcp_agent=True) for _ in range(self.num_hosts)]
@@ -93,17 +89,10 @@ class TestSecurityGroupsSameNetwork(BaseSecurityGroupsSameNetworkTest):
         # because of that using only one host is enough
         ('ovs-hybrid', {
             'firewall_driver': 'iptables_hybrid',
-            'of_interface': 'native',
             'l2_agent_type': constants.AGENT_TYPE_OVS,
             'num_hosts': 1}),
-        ('ovs-openflow-cli', {
+        ('ovs-openflow', {
             'firewall_driver': 'openvswitch',
-            'of_interface': 'ovs-ofctl',
-            'l2_agent_type': constants.AGENT_TYPE_OVS,
-            'num_hosts': 2}),
-        ('ovs-openflow-native', {
-            'firewall_driver': 'openvswitch',
-            'of_interface': 'native',
             'l2_agent_type': constants.AGENT_TYPE_OVS,
             'num_hosts': 2}),
         ('linuxbridge-iptables', {
@@ -518,27 +507,3 @@ class TestSecurityGroupsSameNetwork(BaseSecurityGroupsSameNetworkTest):
                 vm1, vm2, net_helpers.NetcatTester.TCP, port + 1)
             self.verify_no_connectivity_between_vms(
                 vm2, vm1, net_helpers.NetcatTester.TCP, port + 1)
-
-
-class SecurityGroupRulesTest(base.BaseFullStackTestCase):
-
-    def setUp(self):
-        host_descriptions = [environment.HostDescription()]
-        env = environment.Environment(environment.EnvironmentDescription(),
-                                      host_descriptions)
-        super(SecurityGroupRulesTest, self).setUp(env)
-
-    def test_security_group_rule_quota(self):
-        project_id = uuidutils.generate_uuid()
-        quota = self.client.show_quota_details(project_id)
-        sg_rules_used = quota['quota']['security_group_rule']['used']
-        self.assertEqual(0, sg_rules_used)
-
-        self.safe_client.create_security_group(project_id)
-        quota = self.client.show_quota_details(project_id)
-        sg_rules_used = quota['quota']['security_group_rule']['used']
-        self.safe_client.update_quota(project_id, 'security_group_rule',
-                                      sg_rules_used)
-
-        self.assertRaises(nc_exc.OverQuotaClient,
-                          self.safe_client.create_security_group, project_id)
