@@ -14,24 +14,18 @@
 #    under the License.
 
 import datetime
-import os
 import random
-import time
-import warnings
 
 import fixtures
-import mock
 import netaddr
 from neutron_lib import constants
+from neutron_lib.services.logapi import constants as log_const
 from neutron_lib.utils import helpers
 from neutron_lib.utils import net
 from oslo_utils import netutils
 from oslo_utils import timeutils
 import unittest2
 
-from neutron.common import constants as n_const
-from neutron.services.logapi.common import constants as log_const
-
 
 # NOTE(yamahata): from neutron-lib 1.9.1, callback priority was added and
 # priority_group module was added for constants of priority.
@@ -44,42 +38,6 @@ except ImportError as e:
     _CALLBACK_PRIORITY_SUPPORTED = False
 
 
-class WarningsFixture(fixtures.Fixture):
-    """Filters out warnings during test runs."""
-
-    warning_types = (
-        DeprecationWarning, PendingDeprecationWarning, ImportWarning
-    )
-
-    def _setUp(self):
-        self.addCleanup(warnings.resetwarnings)
-        for wtype in self.warning_types:
-            warnings.filterwarnings(
-                "once", category=wtype, module='^neutron\\.')
-
-
-class OpenFixture(fixtures.Fixture):
-    """Mock access to a specific file while preserving open for others."""
-
-    def __init__(self, filepath, contents=''):
-        self.path = filepath
-        self.contents = contents
-
-    def _setUp(self):
-        self.mock_open = mock.mock_open(read_data=self.contents)
-        self._orig_open = open
-
-        def replacement_open(name, *args, **kwargs):
-            if name == self.path:
-                return self.mock_open(name, *args, **kwargs)
-            return self._orig_open(name, *args, **kwargs)
-
-        self._patch = mock.patch('six.moves.builtins.open',
-                                 new=replacement_open)
-        self._patch.start()
-        self.addCleanup(self._patch.stop)
-
-
 class SafeCleanupFixture(fixtures.Fixture):
     """Catch errors in daughter fixture cleanup."""
 
@@ -237,8 +195,8 @@ def get_random_prefixlen(version=4):
     return random.randint(0, maxlen)
 
 
-def get_random_port(start=n_const.PORT_RANGE_MIN):
-    return random.randint(start, n_const.PORT_RANGE_MAX)
+def get_random_port(start=constants.PORT_RANGE_MIN):
+    return random.randint(start, constants.PORT_RANGE_MAX)
 
 
 def get_random_vlan():
@@ -246,25 +204,7 @@ def get_random_vlan():
 
 
 def get_random_ip_version():
-    return random.choice(n_const.IP_ALLOWED_VERSIONS)
-
-
-def get_random_cidr(version=4):
-    if version == 4:
-        return '10.%d.%d.0/%d' % (random.randint(3, 254),
-                                  random.randint(3, 254),
-                                  24)
-    return '2001:db8:%x::/%d' % (random.getrandbits(16), 64)
-
-
-def get_random_EUI():
-    return netaddr.EUI(
-        net.get_random_mac(['fe', '16', '3e', '00', '00', '00'])
-    )
-
-
-def get_random_ip_network(version=4):
-    return netaddr.IPNetwork(get_random_cidr(version=version))
+    return random.choice(constants.IP_ALLOWED_VERSIONS)
 
 
 def get_random_ip_address(version=4):
@@ -282,11 +222,11 @@ def get_random_ip_address(version=4):
 
 
 def get_random_router_status():
-    return random.choice(n_const.VALID_ROUTER_STATUS)
+    return random.choice(constants.VALID_ROUTER_STATUS)
 
 
 def get_random_floatingip_status():
-    return random.choice(n_const.VALID_FLOATINGIP_STATUS)
+    return random.choice(constants.VALID_FLOATINGIP_STATUS)
 
 
 def get_random_flow_direction():
@@ -294,15 +234,15 @@ def get_random_flow_direction():
 
 
 def get_random_ha_states():
-    return random.choice(n_const.VALID_HA_STATES)
+    return random.choice(constants.VALID_HA_STATES)
 
 
 def get_random_ether_type():
-    return random.choice(n_const.VALID_ETHERTYPES)
+    return random.choice(constants.VALID_ETHERTYPES)
 
 
 def get_random_ipam_status():
-    return random.choice(n_const.VALID_IPAM_ALLOCATION_STATUSES)
+    return random.choice(constants.VALID_IPAM_ALLOCATION_STATUSES)
 
 
 def get_random_ip_protocol():
@@ -310,7 +250,7 @@ def get_random_ip_protocol():
 
 
 def get_random_port_binding_statuses():
-    return random.choice(n_const.PORT_BINDING_STATUSES)
+    return random.choice(constants.PORT_BINDING_STATUSES)
 
 
 def get_random_network_segment_range_network_type():
@@ -320,16 +260,6 @@ def get_random_network_segment_range_network_type():
                           constants.TYPE_GENEVE])
 
 
-def reset_random_seed():
-    # reset random seed to make sure other processes extracting values from RNG
-    # don't get the same results (useful especially when you then use the
-    # random values to allocate system resources from global pool, like ports
-    # to listen). Use both current time and pid to make sure no tests started
-    # at the same time get the same values from RNG
-    seed = time.time() + os.getpid()
-    random.seed(seed)
-
-
 def get_random_ipv6_mode():
     return random.choice(constants.IPV6_MODES)
 
