@@ -15,11 +15,13 @@
 
 import mock
 from neutron_lib import constants
+from neutron_lib import exceptions
+from oslo_utils import excutils
 
 from neutron.agent.common import ovs_lib
 from neutron.agent.linux import interface
 from neutron.agent.linux import ip_lib
-from neutron.agent.linux import utils
+from neutron.common import utils
 from neutron.conf.agent import common as config
 from neutron.tests import base
 
@@ -43,7 +45,7 @@ class FakeSubnet(object):
 class FakeAllocation(object):
     subnet = FakeSubnet()
     ip_address = '192.168.1.2'
-    ip_version = 4
+    ip_version = constants.IP_VERSION_4
 
 
 class FakePort(object):
@@ -65,6 +67,9 @@ class TestBase(base.BaseTestCase):
         self.ip = self.ip_p.start()
         self.device_exists_p = mock.patch.object(ip_lib, 'device_exists')
         self.device_exists = self.device_exists_p.start()
+        self.get_devices_with_ip_p = mock.patch.object(ip_lib,
+                                                       'get_devices_with_ip')
+        self.get_devices_with_ip = self.get_devices_with_ip_p.start()
 
 
 class TestABCDriver(TestBase):
@@ -178,9 +183,9 @@ class TestABCDriver(TestBase):
         self.ip_dev.assert_has_calls(expected_calls)
 
     def test_init_router_port_ext_gw_with_dual_stack(self):
-        old_addrs = [dict(ip_version=4, scope='global',
+        old_addrs = [dict(ip_version=constants.IP_VERSION_4, scope='global',
                           dynamic=False, cidr='172.16.77.240/24'),
-                     dict(ip_version=6, scope='global',
+                     dict(ip_version=constants.IP_VERSION_6, scope='global',
                           dynamic=False, cidr='2001:db8:a::123/64')]
         self.ip_dev().addr.list = mock.Mock(return_value=old_addrs)
         self.ip_dev().route.list_onlink_routes.return_value = []
@@ -351,16 +356,18 @@ class TestABCDriver(TestBase):
         addresses = [dict(scope='link',
                           dynamic=False,
                           cidr='fe80:cafe::/64')]
-        self.ip_dev().addr.list = mock.Mock(return_value=addresses)
+        self.get_devices_with_ip.return_value = addresses
         device_name = self.ip_dev().name
         bc = BaseChild(self.conf)
 
         llas = bc.get_ipv6_llas(device_name, ns)
 
         self.assertEqual(addresses, llas)
-        self.ip_dev.assert_has_calls(
-            [mock.call(device_name, namespace=ns),
-             mock.call().addr.list(scope='link', ip_version=6)])
+        kwargs = {'family': utils.get_socket_address_family(
+                                constants.IP_VERSION_6),
+                  'scope': 'link'}
+        self.get_devices_with_ip.assert_called_with(
+            ns, name=device_name, **kwargs)
 
     def test_set_mtu_logs_once(self):
         bc = BaseChild(self.conf)
@@ -458,6 +465,32 @@ class TestOVSInterfaceDriver(TestBase):
 
             self.ip.assert_has_calls(expected)
 
+    def test_plug_new(self):
+        with mock.patch('neutron.agent.ovsdb.impl_idl._connection'):
+            bridge = 'br-int'
+            namespace = '01234567-1234-1234-99'
+            with mock.patch.object(ovs_lib.OVSBridge,
+                                   'delete_port') as delete_port:
+                with mock.patch.object(ovs_lib.OVSBridge, 'replace_port'):
+                    ovs = interface.OVSInterfaceDriver(self.conf)
+                    reraise = mock.patch.object(
+                        excutils, 'save_and_reraise_exception')
+                    reraise.start()
+                    proEr = exceptions.ProcessExecutionError('', 2)
+                    processExecutionError = mock.Mock(side_effect=proEr)
+                    ip = self.ip.return_value
+                    ip.ensure_namespace.side_effect = processExecutionError
+                    ovs.plug_new(
+                        '01234567-1234-1234-99',
+                        'port-1234',
+                        'tap0',
+                        'aa:bb:cc:dd:ee:ff',
+                        bridge=bridge,
+                        namespace=namespace,
+                        prefix='veth',
+                        mtu=9000)
+                    delete_port.assert_called_once_with('tap0')
+
     def test_unplug(self, bridge=None):
         if not bridge:
             bridge = 'br-int'
@@ -534,6 +567,12 @@ class TestOVSInterfaceDriverWithVeth(TestOVSInterfaceDriver):
             root_dev.assert_has_calls([mock.call.link.set_up()])
             ns_dev.assert_has_calls([mock.call.link.set_up()])
 
+    def test_plug_new(self, bridge=None, namespace=None):
+        # The purpose of test_plug_new in parent class(TestOVSInterfaceDriver)
+        # is to test exception(exceptions.ProcessExecutionError), method here
+        # would not go through that code, So just pass
+        pass
+
     def test_unplug(self, bridge=None):
         if not bridge:
             bridge = 'br-int'
@@ -620,71 +659,3 @@ class TestBridgeInterfaceDriver(TestBase):
 
         self.ip_dev.assert_has_calls([mock.call('tap0', namespace=None),
                                       mock.call().link.delete()])
-
-
-class TestIVSInterfaceDriver(TestBase):
-
-    def test_get_device_name(self):
-        br = interface.IVSInterfaceDriver(self.conf)
-        device_name = br.get_device_name(FakePort())
-        self.assertEqual('ns-abcdef01-12', device_name)
-
-    def test_plug_with_prefix(self):
-        self._test_plug(devname='qr-0', prefix='qr-')
-
-    def _test_plug(self, devname=None, namespace=None, prefix=None):
-
-        if not devname:
-            devname = 'ns-0'
-
-        def device_exists(dev, namespace=None):
-            return dev == 'indigo'
-
-        ivs = interface.IVSInterfaceDriver(self.conf)
-        self.device_exists.side_effect = device_exists
-
-        root_dev = mock.Mock()
-        _ns_dev = mock.Mock()
-        ns_dev = mock.Mock()
-        self.ip().add_veth = mock.Mock(return_value=(root_dev, _ns_dev))
-        self.ip().device = mock.Mock(return_value=(ns_dev))
-        expected = [mock.call(), mock.call().add_veth('tap0', devname),
-                    mock.call().device(devname)]
-
-        ivsctl_cmd = ['ivs-ctl', 'add-port', 'tap0']
-
-        with mock.patch.object(utils, 'execute') as execute:
-            ivs.plug('01234567-1234-1234-99',
-                     'port-1234',
-                     devname,
-                     'aa:bb:cc:dd:ee:ff',
-                     namespace=namespace,
-                     prefix=prefix,
-                     mtu=9000)
-            execute.assert_called_once_with(ivsctl_cmd, run_as_root=True)
-
-        ns_dev.assert_has_calls(
-            [mock.call.link.set_address('aa:bb:cc:dd:ee:ff')])
-        ns_dev.assert_has_calls([mock.call.link.set_mtu(9000)])
-        root_dev.assert_has_calls([mock.call.link.set_mtu(9000)])
-        if namespace:
-            expected.extend(
-                [mock.call().ensure_namespace(namespace),
-                 mock.call().ensure_namespace().add_device_to_namespace(
-                     mock.ANY)])
-
-        self.ip.assert_has_calls(expected)
-        root_dev.assert_has_calls([mock.call.link.set_up()])
-        ns_dev.assert_has_calls([mock.call.link.set_up()])
-
-    def test_plug_namespace(self):
-        self._test_plug(namespace='mynamespace')
-
-    def test_unplug(self):
-        ivs = interface.IVSInterfaceDriver(self.conf)
-        ivsctl_cmd = ['ivs-ctl', 'del-port', 'tap0']
-        with mock.patch.object(utils, 'execute') as execute:
-            ivs.unplug('ns-0')
-            execute.assert_called_once_with(ivsctl_cmd, run_as_root=True)
-            self.ip_dev.assert_has_calls([mock.call('ns-0', namespace=None),
-                                          mock.call().link.delete()])
