@@ -17,9 +17,9 @@ import datetime
 
 import mock
 from neutron_lib.api.definitions import dhcpagentscheduler as das_apidef
-from neutron_lib.api.definitions import portbindings
 from neutron_lib import constants
 from neutron_lib import context
+from neutron_lib.db import api as db_api
 from neutron_lib.plugins import constants as plugin_constants
 from neutron_lib.plugins import directory
 from neutron_lib import rpc as n_rpc
@@ -684,22 +684,20 @@ class OvsAgentSchedulerTestCase(OvsAgentSchedulerTestCaseBase):
 
     def _take_down_agent_and_run_reschedule(self, host):
         # take down the agent on host A and ensure B is alive
-        self.adminContext.session.begin(subtransactions=True)
-        query = self.adminContext.session.query(agent_model.Agent)
-        agt = query.filter_by(host=host).first()
-        agt.heartbeat_timestamp = (
-            agt.heartbeat_timestamp - datetime.timedelta(hours=1))
-        self.adminContext.session.commit()
+        with db_api.CONTEXT_WRITER.using(self.adminContext):
+            query = self.adminContext.session.query(agent_model.Agent)
+            agt = query.filter_by(host=host).first()
+            agt.heartbeat_timestamp = (
+                agt.heartbeat_timestamp - datetime.timedelta(hours=1))
 
         plugin = directory.get_plugin(plugin_constants.L3)
         plugin.reschedule_routers_from_down_agents()
 
     def _set_agent_admin_state_up(self, host, state):
-        self.adminContext.session.begin(subtransactions=True)
-        query = self.adminContext.session.query(agent_model.Agent)
-        agt_db = query.filter_by(host=host).first()
-        agt_db.admin_state_up = state
-        self.adminContext.session.commit()
+        with db_api.CONTEXT_WRITER.using(self.adminContext):
+            query = self.adminContext.session.query(agent_model.Agent)
+            agt_db = query.filter_by(host=host).first()
+            agt_db.admin_state_up = state
 
     def test_router_rescheduler_catches_rpc_db_and_reschedule_exceptions(self):
         with self.router():
@@ -1070,44 +1068,6 @@ class OvsAgentSchedulerTestCase(OvsAgentSchedulerTestCaseBase):
                 # No router will be auto scheduled.
                 self.assertEqual(0, len(host_routers['routers']))
 
-    def test_sync_dvr_router_with_fixedip_on_fip_net(self):
-        l3_rpc_cb = l3_rpc.L3RpcCallback()
-        self._register_dvr_agents()
-
-        with self.subnet() as s:
-            # first create an external network
-            net_id = s['subnet']['network_id']
-            self._set_net_external(net_id)
-            # create router with external gateway
-            router = {'name': 'router1',
-                      'external_gateway_info': {'network_id': net_id},
-                      'tenant_id': 'tenant_id',
-                      'admin_state_up': True,
-                      'distributed': True}
-            r = self.l3plugin.create_router(self.adminContext,
-                                            {'router': router})
-            self.l3plugin.schedule_router(self.adminContext, r['id'])
-            with self.port(subnet=s,
-                           device_owner=DEVICE_OWNER_COMPUTE) as port:
-                # bind port to L3_HOSTB
-                updated_port = {
-                    "port": {
-                        portbindings.HOST_ID: L3_HOSTB
-                    }
-                }
-                self.plugin.update_port(
-                    self.adminContext,
-                    port['port']['id'],
-                    updated_port
-                )
-                ret_b = l3_rpc_cb.sync_routers(
-                    self.adminContext,
-                    host=L3_HOSTB,
-                    router_ids=[r['id']])
-
-                router_ids = [r['id'] for r in ret_b]
-                self.assertEqual(0, len(router_ids))
-
     def test_router_without_l3_agents(self):
         with self.subnet() as s:
             self._set_net_external(s['subnet']['network_id'])
