@@ -88,7 +88,6 @@ class TunnelTest(object):
                              'neutron.agent.firewall.NoopFirewallDriver',
                              group='SECURITYGROUP')
         cfg.CONF.set_override('report_interval', 0, 'AGENT')
-        cfg.CONF.set_override('explicitly_egress_direct', True, 'AGENT')
 
         self.INT_BRIDGE = 'integration_bridge'
         self.TUN_BRIDGE = 'tunnel_bridge'
@@ -196,7 +195,7 @@ class TunnelTest(object):
             '_check_bridge_datapath_id').start()
         self._define_expected_calls()
 
-    def _define_expected_calls(self, arp_responder=False, igmp_snooping=False):
+    def _define_expected_calls(self, arp_responder=False):
         self.mock_int_bridge_cls_expected = [
             mock.call(self.INT_BRIDGE,
                       datapath_type=mock.ANY),
@@ -215,7 +214,6 @@ class TunnelTest(object):
             mock.call.create(),
             mock.call.set_secure_mode(),
             mock.call.setup_controllers(mock.ANY),
-            mock.call.set_igmp_snooping_state(igmp_snooping),
             mock.call.setup_default_table(),
         ]
 
@@ -284,20 +282,6 @@ class TunnelTest(object):
         self.intb_expected = []
         self.execute_expected = []
 
-        self.mock_int_bridge_expected += [
-            mock.call.install_goto(
-                dest_table_id=constants.LOCAL_MAC_DIRECT,
-                in_port=self.MAP_TUN_INT_OFPORT,
-                priority=4, table_id=constants.TRANSIENT_TABLE),
-            mock.call.install_goto(
-                dest_table_id=constants.LOCAL_MAC_DIRECT,
-                in_port=self.TUN_OFPORT,
-                priority=4, table_id=constants.TRANSIENT_TABLE),
-            mock.call.install_goto(
-                dest_table_id=constants.TRANSIENT_EGRESS_TABLE,
-                table_id=constants.LOCAL_MAC_DIRECT),
-        ]
-
     def _build_agent(self, **config_opts_agent):
         """Configure and initialize OVS agent.
 
@@ -365,13 +349,7 @@ class TunnelTest(object):
     #                 The next two tests use l2_pop flag to test ARP responder
     def test_construct_with_arp_responder(self):
         self._build_agent(l2_population=True, arp_responder=True)
-        self._define_expected_calls(arp_responder=True)
-        self._verify_mock_calls()
-
-    def test_construct_with_igmp_snooping(self):
-        cfg.CONF.set_override('igmp_snooping_enable', True, 'OVS')
-        self._build_agent()
-        self._define_expected_calls(igmp_snooping=True)
+        self._define_expected_calls(True)
         self._verify_mock_calls()
 
     def test_construct_without_arp_responder(self):
@@ -598,14 +576,8 @@ class TunnelTest(object):
 
         self.mock_int_bridge_expected += [
             mock.call.check_canary_table(),
-            mock.call.deferred(full_ordered=True, use_bundle=True),
-            mock.call.deferred().__enter__(),
-            mock.call.deferred().__exit__(None, None, None),
             mock.call.cleanup_flows(),
-            mock.call.check_canary_table(),
-            mock.call.deferred(full_ordered=True, use_bundle=True),
-            mock.call.deferred().__enter__(),
-            mock.call.deferred().__exit__(None, None, None),
+            mock.call.check_canary_table()
         ]
         self.mock_map_tun_bridge_expected += [
             mock.call.cleanup_flows(),
@@ -676,10 +648,6 @@ class TunnelTest(object):
             self._verify_mock_calls()
 
 
-class TunnelTestOFCtl(TunnelTest, ovs_test_base.OVSOFCtlTestBase):
-    pass
-
-
 class TunnelTestOSKen(TunnelTest, ovs_test_base.OVSOSKenTestBase):
     pass
 
@@ -687,7 +655,7 @@ class TunnelTestOSKen(TunnelTest, ovs_test_base.OVSOSKenTestBase):
 class TunnelTestUseVethInterco(TunnelTest):
     USE_VETH_INTERCONNECTION = True
 
-    def _define_expected_calls(self, arp_responder=False, igmp_snooping=False):
+    def _define_expected_calls(self, arp_responder=False):
         self.mock_int_bridge_cls_expected = [
             mock.call(self.INT_BRIDGE,
                       datapath_type=mock.ANY),
@@ -705,7 +673,6 @@ class TunnelTestUseVethInterco(TunnelTest):
             mock.call.create(),
             mock.call.set_secure_mode(),
             mock.call.setup_controllers(mock.ANY),
-            mock.call.set_igmp_snooping_state(igmp_snooping),
             mock.call.setup_default_table(),
         ]
 
@@ -773,25 +740,6 @@ class TunnelTestUseVethInterco(TunnelTest):
         self.execute_expected = [mock.call(['udevadm', 'settle',
                                             '--timeout=10'])]
 
-        self.mock_int_bridge_expected += [
-            mock.call.install_goto(
-                dest_table_id=constants.LOCAL_MAC_DIRECT,
-                in_port=self.MAP_TUN_INT_OFPORT,
-                priority=4, table_id=constants.TRANSIENT_TABLE),
-            mock.call.install_goto(
-                dest_table_id=constants.LOCAL_MAC_DIRECT,
-                in_port=self.TUN_OFPORT,
-                priority=4, table_id=constants.TRANSIENT_TABLE),
-            mock.call.install_goto(
-                dest_table_id=constants.TRANSIENT_EGRESS_TABLE,
-                table_id=constants.LOCAL_MAC_DIRECT),
-        ]
-
-
-class TunnelTestUseVethIntercoOFCtl(TunnelTestUseVethInterco,
-                                    ovs_test_base.OVSOFCtlTestBase):
-    pass
-
 
 class TunnelTestUseVethIntercoOSKen(TunnelTestUseVethInterco,
                                   ovs_test_base.OVSOSKenTestBase):
@@ -801,18 +749,12 @@ class TunnelTestUseVethIntercoOSKen(TunnelTestUseVethInterco,
 class TunnelTestWithMTU(TunnelTestUseVethInterco):
     VETH_MTU = 1500
 
-    def _define_expected_calls(self, arp_responder=False, igmp_snooping=False):
-        super(TunnelTestWithMTU, self)._define_expected_calls(
-            arp_responder, igmp_snooping)
+    def _define_expected_calls(self, arp_responder=False):
+        super(TunnelTestWithMTU, self)._define_expected_calls(arp_responder)
         self.inta_expected.append(mock.call.link.set_mtu(self.VETH_MTU))
         self.intb_expected.append(mock.call.link.set_mtu(self.VETH_MTU))
 
 
-class TunnelTestWithMTUOFCtl(TunnelTestWithMTU,
-                             ovs_test_base.OVSOFCtlTestBase):
-    pass
-
-
 class TunnelTestWithMTUOSKen(TunnelTestWithMTU,
                            ovs_test_base.OVSOSKenTestBase):
     pass
