@@ -32,7 +32,6 @@ from oslo_context import context as oslo_context
 from sqlalchemy.orm import exc
 
 from neutron.agent import rpc as agent_rpc
-from neutron.common import constants as n_const
 from neutron.db import provisioning_blocks
 from neutron.plugins.ml2 import db as ml2_db
 from neutron.plugins.ml2.drivers import type_tunnel
@@ -63,7 +62,7 @@ class RpcCallbacksTestCase(base.BaseTestCase):
         }
         with mock.patch('neutron.plugins.ml2.plugin.Ml2Plugin'
                         '._device_to_port_id'),\
-            mock.patch.object(self.callbacks, 'notify_l2pop_port_wiring'):
+                mock.patch.object(self.callbacks, 'notify_l2pop_port_wiring'):
             with mock.patch('neutron.db.provisioning_blocks.'
                             'provisioning_complete') as pc:
                 self.callbacks.update_device_up(mock.Mock(), **kwargs)
@@ -162,7 +161,7 @@ class RpcCallbacksTestCase(base.BaseTestCase):
         port['device_owner'] = constants.DEVICE_OWNER_COMPUTE_PREFIX
         port[portbindings.HOST_ID] = 'other-host'
         res = self.callbacks.get_device_details(mock.Mock(), host='host')
-        self.assertIn(n_const.NO_ACTIVE_BINDING, res)
+        self.assertIn(constants.NO_ACTIVE_BINDING, res)
 
     def test_get_device_details_qos_policy_id_from_port(self):
         port = collections.defaultdict(
@@ -215,6 +214,17 @@ class RpcCallbacksTestCase(base.BaseTestCase):
             self.callbacks.get_devices_details_list_and_failed_devices)
         self._test_get_devices_list(callback, devices, expected)
 
+    def test_get_network_details(self):
+        kwargs = {'agent_id': 'agent_id',
+                  'host': 'host_id',
+                  'network': 'network'}
+        with mock.patch.object(self.plugin, 'get_network') as mock_get_network:
+            mock_get_network.return_value = 'net_details'
+            self.assertEqual(
+                'net_details',
+                self.callbacks.get_network_details('fake_context', **kwargs))
+            mock_get_network.assert_called_once_with('fake_context', 'network')
+
     def test_get_devices_details_list_and_failed_devices_empty_dev(self):
         with mock.patch.object(self.callbacks, 'get_device_details') as f:
             res = self.callbacks.get_devices_details_list_and_failed_devices(
@@ -240,6 +250,12 @@ class RpcCallbacksTestCase(base.BaseTestCase):
             (self.plugin.nova_notifier.notify_port_active_direct.
              assert_called_once_with(port))
 
+    def test_update_device_up_with_device_not_bound_to_host_no_notify(self):
+        cfg.CONF.set_override('notify_nova_on_port_status_changes', False)
+        self.assertIsNone(self._test_update_device_not_bound_to_host(
+            self.callbacks.update_device_up))
+        self.plugin.nova_notifier.notify_port_active_direct.assert_not_called()
+
     def test_update_device_down_with_device_not_bound_to_host(self):
         self.assertEqual(
             {'device': 'fake_device', 'exists': True},
@@ -259,16 +275,6 @@ class RpcCallbacksTestCase(base.BaseTestCase):
             mock.ANY, 'fake_port_id', constants.PORT_STATUS_DOWN,
             'fake_host')
 
-    def test_notify_l2pop_port_wiring_non_dvr_port(self):
-        port = {'device_owner': constants.DEVICE_OWNER_COMPUTE_PREFIX}
-        l2pop_driver = (
-            self.plugin.mechanism_manager.mech_drivers.get.return_value)
-        with mock.patch.object(ml2_db, 'get_port') as ml2_db_get_port:
-            ml2_db_get_port.return_value = port
-            self.callbacks.notify_l2pop_port_wiring(
-                'port_id', mock.Mock(), 'DOWN', 'host', agent_restarted=None)
-            self.assertFalse(l2pop_driver.obj.agent_restarted.called)
-
     def test_update_device_down_call_update_port_status_failed(self):
         self.plugin.update_port_status.side_effect = exc.StaleDataError
         self.assertEqual({'device': 'fake_device', 'exists': False},
@@ -346,7 +352,7 @@ class RpcApiTestCase(base.BaseTestCase):
             expected = 'foo'
 
         ctxt = oslo_context.RequestContext(user_id='fake_user',
-                                           tenant='fake_project')
+                                           project_id='fake_project')
         expected_retval = expected if rpc_method == 'call' else None
         expected_version = kwargs.pop('version', None)
         fanout = kwargs.pop('fanout', False)
@@ -494,3 +500,12 @@ class RpcApiTestCase(base.BaseTestCase):
                            devices=['fake_device1', 'fake_device2'],
                            agent_id='fake_agent_id', host='fake_host',
                            version='1.5')
+
+    def test_get_ports_by_vnic_type_and_host(self):
+        rpcapi = agent_rpc.PluginApi(topics.PLUGIN)
+        self._test_rpc_api(rpcapi, None,
+                           'get_ports_by_vnic_type_and_host',
+                           rpc_method='call',
+                           vnic_type='fake_device1',
+                           host='fake_host',
+                           version='1.7')
